// Copyright 2022 Bitcoding

package com.btcpiyush.ads.google_applovin_unity_ads;

import android.app.Activity;
import android.graphics.Color;
import android.graphics.drawable.GradientDrawable;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;

import com.applovin.mediation.nativeAds.MaxNativeAdView;
import com.applovin.mediation.nativeAds.MaxNativeAdViewBinder;
import com.btcpiyush.ads.google_applovin_unity_ads.GoogleApplovinUnityAdsPlugin.NativeAdFactory;
import com.facebook.ads.AdOptionsView;
import com.facebook.ads.NativeAdLayout;
import com.google.android.gms.ads.nativead.NativeAd;
import com.google.android.gms.ads.nativead.NativeAdView;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class CustomNativeAd implements NativeAdFactory {
    private final LayoutInflater layoutInflater;

    public CustomNativeAd(LayoutInflater layoutInflater) {
        this.layoutInflater = layoutInflater;
    }

    @Override
    public NativeAdView createNativeAd(NativeAd nativeAd, Map<String, Object> customOptions) {
        final NativeAdView adView = (NativeAdView) layoutInflater.inflate(R.layout.native_full, null);

        // Set the media view.
        adView.setMediaView(adView.findViewById(R.id.media_view1));

        // Set other ad assets.
        adView.setHeadlineView(adView.findViewById(R.id.primary));
        adView.setBodyView(adView.findViewById(R.id.body_text));
        adView.setCallToActionView(adView.findViewById(R.id.cta));
        adView.setIconView(adView.findViewById(R.id.icon));
//    adView.setPriceView(adView.findViewById(R.id.ad_price));
//    adView.setStarRatingView(adView.findViewById(R.id.ad_stars));
        adView.setStoreView(adView.findViewById(R.id.body));
//    adView.setAdvertiserView(adView.findViewById(R.id.ad_advertiser));

        // The headline and mediaContent are guaranteed to be in every NativeAd.
        ((TextView) adView.getHeadlineView()).setText(nativeAd.getHeadline());
        if (nativeAd.getMediaContent() == null) {
            adView.getMediaView().setVisibility(View.GONE);
        } else {
            adView.getMediaView().setVisibility(View.VISIBLE);
            adView.getMediaView().setMediaContent(nativeAd.getMediaContent());
        }

        // These assets aren't guaranteed to be in every NativeAd, so it's important to
        // check before trying to display them.
        if (nativeAd.getBody() == null) {
            adView.getBodyView().setVisibility(View.INVISIBLE);
        } else {
            adView.getBodyView().setVisibility(View.VISIBLE);
            ((TextView) adView.getBodyView()).setText(nativeAd.getBody());
        }

        if (nativeAd.getCallToAction() == null) {
            adView.getCallToActionView().setVisibility(View.INVISIBLE);
        } else {
            adView.getCallToActionView().setVisibility(View.VISIBLE);
            Button btn = ((Button) adView.getCallToActionView());
            btn.setText(nativeAd.getCallToAction());
            // Color.parseColor() method allow us to convert
            applyGradient(btn, customOptions);

        }

        if (nativeAd.getIcon() == null) {
            adView.getIconView().setVisibility(View.GONE);
        } else {
            ((ImageView) adView.getIconView()).setImageDrawable(nativeAd.getIcon().getDrawable());
            adView.getIconView().setVisibility(View.VISIBLE);
        }

   /* if (nativeAd.getPrice() == null) {
      adView.getPriceView().setVisibility(View.INVISIBLE);
    } else {
      adView.getPriceView().setVisibility(View.VISIBLE);
      ((TextView) adView.getPriceView()).setText(nativeAd.getPrice());
    }*/

        if (nativeAd.getStore() == null) {
            adView.getStoreView().setVisibility(View.INVISIBLE);
        } else {
            adView.getStoreView().setVisibility(View.VISIBLE);
            ((TextView) adView.getStoreView()).setText(nativeAd.getStore());
        }

    /*if (nativeAd.getStarRating() == null) {
      adView.getStarRatingView().setVisibility(View.INVISIBLE);
    } else {
      ((RatingBar) adView.getStarRatingView()).setRating(nativeAd.getStarRating().floatValue());
      adView.getStarRatingView().setVisibility(View.VISIBLE);
    }*/

   /* if (nativeAd.getAdvertiser() == null) {
      adView.getAdvertiserView().setVisibility(View.INVISIBLE);
    } else {
      adView.getAdvertiserView().setVisibility(View.VISIBLE);
      ((TextView) adView.getAdvertiserView()).setText(nativeAd.getAdvertiser());
    }*/

        // This method tells the Google Mobile Ads SDK that you have finished populating your
        // native ad view with this native ad.
        adView.setNativeAd(nativeAd);

        return adView;
    }

    @Override
    public NativeAdView createMediumNativeAd(NativeAd nativeAd, Map<String, Object> customOptions) {
        final NativeAdView adView = (NativeAdView) layoutInflater.inflate(R.layout.native_medium, null);

        // Set other ad assets.
        adView.setHeadlineView(adView.findViewById(R.id.primary));
        adView.setBodyView(adView.findViewById(R.id.body_text));
        adView.setCallToActionView(adView.findViewById(R.id.cta));
        adView.setIconView(adView.findViewById(R.id.icon));
        adView.setStoreView(adView.findViewById(R.id.body));


        ((TextView) adView.getHeadlineView()).setText(nativeAd.getHeadline());

        if (nativeAd.getBody() == null) {
            adView.getBodyView().setVisibility(View.INVISIBLE);
        } else {
            adView.getBodyView().setVisibility(View.VISIBLE);
            ((TextView) adView.getBodyView()).setText(nativeAd.getBody());
        }

        if (nativeAd.getCallToAction() == null) {
            adView.getCallToActionView().setVisibility(View.INVISIBLE);
        } else {
            adView.getCallToActionView().setVisibility(View.VISIBLE);
            Button btn = ((Button) adView.getCallToActionView());
            btn.setText(nativeAd.getCallToAction());
            // Color.parseColor() method allow us to convert
            applyGradient(btn, customOptions);
        }

        if (nativeAd.getIcon() == null) {
            adView.getIconView().setVisibility(View.GONE);
        } else {
            ((ImageView) adView.getIconView()).setImageDrawable(nativeAd.getIcon().getDrawable());
            adView.getIconView().setVisibility(View.VISIBLE);
        }

        if (nativeAd.getStore() == null) {
            adView.getStoreView().setVisibility(View.INVISIBLE);
        } else {
            adView.getStoreView().setVisibility(View.VISIBLE);
            ((TextView) adView.getStoreView()).setText(nativeAd.getStore());
        }

        adView.setNativeAd(nativeAd);

        return adView;
    }

    @Override
    public NativeAdLayout createFacebookMediumNativeAd(@NonNull Activity context, com.facebook.ads.NativeAd nativeAd, Map<String, Object> customOptions) {
        nativeAd.unregisterView();
        final NativeAdLayout nativeAdLayout = (NativeAdLayout) layoutInflater.inflate(R.layout.native_facebook_ads_layout, null);
        LayoutInflater inflater = LayoutInflater.from(context);
        // Inflate the Ad view.  The layout referenced should be the one you created in the last step.
        final LinearLayout adView = (LinearLayout) inflater.inflate(R.layout.native_facebook_ads, nativeAdLayout, false);
        nativeAdLayout.addView(adView);

        // Add the AdOptionsView
        LinearLayout adChoicesContainer = adView.findViewById(R.id.ad_choices_container);
        AdOptionsView adOptionsView = new AdOptionsView(context, nativeAd, nativeAdLayout);
        adChoicesContainer.removeAllViews();
        adChoicesContainer.addView(adOptionsView, 0);

        // Create native UI using the ad metadata.
        com.facebook.ads.MediaView nativeAdIcon = adView.findViewById(R.id.native_ad_icon);
        TextView nativeAdTitle = adView.findViewById(R.id.native_ad_title);
        com.facebook.ads.MediaView nativeAdMedia = adView.findViewById(R.id.native_ad_media);
        TextView nativeAdSocialContext = adView.findViewById(R.id.native_ad_social_context);
        TextView nativeAdBody = adView.findViewById(R.id.native_ad_body);
        TextView sponsoredLabel = adView.findViewById(R.id.native_ad_sponsored_label);
        Button nativeAdCallToAction = adView.findViewById(R.id.native_ad_call_to_action);
        applyGradient(nativeAdCallToAction, customOptions);
        nativeAdMedia.setVisibility(View.GONE);
        // Set the Text.
        nativeAdTitle.setText(nativeAd.getAdvertiserName());
        nativeAdBody.setText(nativeAd.getAdBodyText());
        nativeAdSocialContext.setText(nativeAd.getAdSocialContext());
        nativeAdCallToAction.setVisibility(nativeAd.hasCallToAction() ? View.VISIBLE : View.INVISIBLE);
        nativeAdCallToAction.setText(nativeAd.getAdCallToAction());
        sponsoredLabel.setText(nativeAd.getSponsoredTranslation());

        // Create a list of clickable views
        List<View> clickableViews = new ArrayList<>();
        clickableViews.add(nativeAdTitle);
        clickableViews.add(nativeAdCallToAction);

        // Register the Title and CTA button to listen for clicks.
        nativeAd.registerViewForInteraction(adView, nativeAdMedia, nativeAdIcon, clickableViews);

        return nativeAdLayout;
    }

    @Override
    public NativeAdLayout createFacebookNativeAd(@NonNull Activity context, com.facebook.ads.NativeAd nativeAd, Map<String, Object> customOptions) {
        nativeAd.unregisterView();
        final NativeAdLayout nativeAdLayout = (NativeAdLayout) layoutInflater.inflate(R.layout.native_facebook_ads_layout, null);
        LayoutInflater inflater = LayoutInflater.from(context);
        // Inflate the Ad view.  The layout referenced should be the one you created in the last step.
        final LinearLayout adView = (LinearLayout) inflater.inflate(R.layout.native_facebook_ads, nativeAdLayout, false);
        nativeAdLayout.addView(adView);

        // Add the AdOptionsView
        LinearLayout adChoicesContainer = adView.findViewById(R.id.ad_choices_container);
        AdOptionsView adOptionsView = new AdOptionsView(context, nativeAd, nativeAdLayout);
        adChoicesContainer.removeAllViews();
        adChoicesContainer.addView(adOptionsView, 0);

        // Create native UI using the ad metadata.
        com.facebook.ads.MediaView nativeAdIcon = adView.findViewById(R.id.native_ad_icon);
        TextView nativeAdTitle = adView.findViewById(R.id.native_ad_title);
        com.facebook.ads.MediaView nativeAdMedia = adView.findViewById(R.id.native_ad_media);
        TextView nativeAdSocialContext = adView.findViewById(R.id.native_ad_social_context);
        TextView nativeAdBody = adView.findViewById(R.id.native_ad_body);
        TextView sponsoredLabel = adView.findViewById(R.id.native_ad_sponsored_label);
        Button nativeAdCallToAction = adView.findViewById(R.id.native_ad_call_to_action);
        applyGradient(nativeAdCallToAction, customOptions);
        // Set the Text.
        nativeAdTitle.setText(nativeAd.getAdvertiserName());
        nativeAdBody.setText(nativeAd.getAdBodyText());
        nativeAdSocialContext.setText(nativeAd.getAdSocialContext());
        nativeAdCallToAction.setVisibility(nativeAd.hasCallToAction() ? View.VISIBLE : View.INVISIBLE);
        nativeAdCallToAction.setText(nativeAd.getAdCallToAction());
        sponsoredLabel.setText(nativeAd.getSponsoredTranslation());

        // Create a list of clickable views
        List<View> clickableViews = new ArrayList<>();
        clickableViews.add(nativeAdTitle);
        clickableViews.add(nativeAdCallToAction);

        // Register the Title and CTA button to listen for clicks.
        nativeAd.registerViewForInteraction(adView, nativeAdMedia, nativeAdIcon, clickableViews);

        return nativeAdLayout;
    }

    @Override
    public MaxNativeAdView createAppLovinNativeAd(@NonNull Activity context, Map<String, Object> customOptions) {
        View view = context.getLayoutInflater().inflate(R.layout.native_full_template, null);

        applyGradient((Button) view.findViewById(R.id.cta), customOptions);

        MaxNativeAdViewBinder binder = new MaxNativeAdViewBinder.Builder(view)
                        .setTitleTextViewId(R.id.primary)
                        .setBodyTextViewId(R.id.body_text)
                        .setAdvertiserTextViewId(R.id.body)
                        .setIconImageViewId(R.id.icon)
                        .setMediaContentViewGroupId(R.id.media_view)
                        .setCallToActionButtonId(R.id.cta)
                        .build();

        return new MaxNativeAdView(binder, context);
    }

    @Override
    public MaxNativeAdView createAppLovinMediumNativeAd(@NonNull Activity context, Map<String, Object> customOptions) {
        View view = context.getLayoutInflater().inflate(R.layout.native_medium_template, null);

        applyGradient((Button) view.findViewById(R.id.cta), customOptions);

        MaxNativeAdViewBinder binder =
                new MaxNativeAdViewBinder.Builder(view)
                        .setTitleTextViewId(R.id.primary)
                        .setBodyTextViewId(R.id.body_text)
                        .setAdvertiserTextViewId(R.id.body)
                        .setIconImageViewId(R.id.icon)
                        .setCallToActionButtonId(R.id.cta)
                        .build();

        return new MaxNativeAdView(binder, context);
    }

    private void applyGradient(Button btn, Map<String, Object> customOptions) {
        if (customOptions != null) {
            if (!customOptions.isEmpty()) {
                final String defaultColor = "#000000";
                GradientDrawable gd = null;
                GradientDrawable.Orientation orientation = GradientDrawable.Orientation.LEFT_RIGHT;
                if (customOptions.containsKey("btn_gradient_orientation") && customOptions.get("btn_gradient_orientation") != null && customOptions.get("btn_gradient_orientation") instanceof Integer) {
                    try {
                        switch ((int) customOptions.get("btn_gradient_orientation")) {
                            case 0:
                                orientation = GradientDrawable.Orientation.TOP_BOTTOM;
                                break;
                            case 1:
                                orientation = GradientDrawable.Orientation.TR_BL;
                                break;
                            case 2:
                                orientation = GradientDrawable.Orientation.RIGHT_LEFT;
                                break;
                            case 3:
                                orientation = GradientDrawable.Orientation.BR_TL;
                                break;
                            case 4:
                                orientation = GradientDrawable.Orientation.BOTTOM_TOP;
                                break;
                            case 5:
                                orientation = GradientDrawable.Orientation.BL_TR;
                                break;
                            case 6:
                            default:
                                orientation = GradientDrawable.Orientation.LEFT_RIGHT;
                                break;
                            case 7:
                                orientation = GradientDrawable.Orientation.TL_BR;
                                break;
                        }
                    } catch (Exception e) { }
                }
                if (customOptions.containsKey("btn_colors") && customOptions.get("btn_colors") != null && customOptions.get("btn_colors") instanceof ArrayList) {
                    try {
                        ArrayList<String> colorsList = (ArrayList<String>) customOptions.get("btn_colors");
                        if (colorsList.size() == 1) {
                            colorsList.add( colorsList.get(0));
                        }

                        if (colorsList.size() == 0) {
                            colorsList.add(defaultColor);
                            colorsList.add(defaultColor);
                        }

                        int[] colors = new int[colorsList.size()];
                        for (int i = 0; i < colorsList.size(); i++) {
                            try {
                                if (colorsList.get(i) != null && !colorsList.get(i).isEmpty()) {
                                    colors[i] = Color.parseColor(colorsList.get(i));
                                }
                            } catch (Exception e) {
                                colors[i] = Color.parseColor(defaultColor);
                            }
                        }

                        //create a new gradient color
                        gd = new GradientDrawable(orientation, colors);
                    } catch (Exception e) {}
                }
                if (customOptions.containsKey("btn_corner_radius") && customOptions.get("btn_corner_radius") != null && !customOptions.get("btn_corner_radius").toString().isEmpty()) {
                    try {
                        if (gd == null) {
                            int[] colors = {Color.parseColor(defaultColor), Color.parseColor(defaultColor)};
                            gd = new GradientDrawable(orientation, colors);
                        }
                        gd.setCornerRadius(Float.parseFloat(customOptions.get("btn_corner_radius").toString()));
                    } catch (Exception e) { }
                }
                if (customOptions.containsKey("btn_text_color") && customOptions.get("btn_text_color") != null && !customOptions.get("btn_text_color").toString().isEmpty()) {
                    try {
                        if (gd == null) {
                            int[] colors = {Color.parseColor(defaultColor), Color.parseColor(defaultColor)};
                            gd = new GradientDrawable(orientation, colors);
                        }
                        btn.setTextColor(Color.parseColor(customOptions.get("btn_text_color").toString()));
                    } catch (Exception e) { }
                }
                try {
                    if (gd != null) {
                        btn.setBackground(gd);
                    }
                } catch (Exception e) {}
            }
        }
    }

}