// Copyright 2023 Bitcoding
// See the License for the specific language governing permissions and
// limitations under the License.

package com.btcpiyush.ads.google_applovin_unity_ads;

import android.app.Activity;
import android.app.Dialog;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.provider.Browser;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.WindowManager;
import android.webkit.WebChromeClient;
import android.webkit.WebResourceRequest;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.webkit.WebViewClient;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.browser.customtabs.CustomTabsIntent;

import com.btcpiyush.ads.google_applovin_unity_ads.databinding.CustomInterstitialAdsBinding;
import com.google.android.gms.ads.LoadAdError;
import com.squareup.picasso.Picasso;
import com.squareup.picasso.Target;

import org.json.JSONException;
import org.json.JSONObject;

import java.lang.ref.WeakReference;

class FlutterCustomInterstitialAd extends FlutterAd.FlutterOverlayAd {
    private static final String TAG = "FlutterInterstitialAd";
    @NonNull
    private final AdInstanceManager manager;
    @NonNull
    private final String adUnitId;
    @Nullable
    private CustomAdDialog ad;
    @Nullable
    private JSONObject model;
    @NonNull
    private final FlutterAdLoader flutterAdLoader;

    public FlutterCustomInterstitialAd(
            int adId,
            @NonNull AdInstanceManager manager,
            @NonNull String adUnitId,
            @Nullable String model,
            @NonNull FlutterAdLoader flutterAdLoader) {
        super(adId);
        this.manager = manager;
        this.adUnitId = adUnitId;
        try {
            this.model = new JSONObject(model);
        } catch (Exception e) {
            e.printStackTrace();
            this.model = null;
        }
        this.flutterAdLoader = flutterAdLoader;
    }

    @Override
    void load() {
        if (manager != null && adUnitId != null) {
            Log.e(TAG, "load:    Step 1" );
            ad = new CustomAdDialog(manager.getActivity(), adUnitId,this.model, new DelegatingInterstitialAdLoadCallback(this));
            ad.setCancelable(false);
            ad.setCanceledOnTouchOutside(false);
            Log.e(TAG, "load:    Step 2" );
            try {
                if (model == null) {
                    ad.loadAd();
                } else {
                    if (this.model.has("is_qureka_custom_tab") && this.model.getBoolean("is_qureka_custom_tab")) {
                        if(this.model.has("is_qureka_imageview") && this.model.getBoolean("is_qureka_imageview")) {
                            ad.loadAd();
                        } else {
                            if (this.model.has("qureka_link") && this.model.getString("qureka_link") != null && !this.model.getString("qureka_link").isEmpty()) {
                                ad.isCustomChrome(true);
                                onAdLoaded(null);
                            } else {
                                ad.loadAd();
                            }
                        }
                    } else {
                        if (this.model.has("is_qureka_imageview") && this.model.getBoolean("is_qureka_imageview")) {
                            ad.loadAd();
                        } else if (this.model.has("is_qureka_webview") && this.model.getBoolean("is_qureka_webview")) {
                            onAdLoaded(null);
                        } else {
                            ad.loadAd();
                        }
                    }
                }
            } catch (Exception e) {
                Log.e(TAG, "load:    Step 5" );
                ad.loadAd();
            }
        }
    }

    void onAdLoaded(Bitmap bitmap) {
        ad.setBitmap(bitmap);
        manager.onAdLoaded(adId, null);
    }

    void onAdFailedToLoad(LoadAdError loadAdError) {
        Log.e(TAG, "onAdFailedToLoad:   step 20" );
        ad = null;
        manager.onAdFailedToLoad(adId, new FlutterLoadAdError(loadAdError));
    }

    @Override
    void dispose() {
        if (ad != null)
           ad = null;
    }

    private void onFailedToShow() {
        manager.onFailedToShowFullScreenContent(adId, new com.google.android.gms.ads.AdError(121, "Error showing interstitial - the interstitial ad wasn't loaded yet.","121"));
    }

    @Override
    public void show() {
        if (ad == null) {
            onFailedToShow();
            Log.e(TAG, "Error showing interstitial - the interstitial ad wasn't loaded yet.");
            return;
        }
        ad.show();
    }

    @Override
    public void setImmersiveMode(boolean immersiveModeEnabled) {

    }

    private static final class CustomAdDialog extends Dialog {
        private CustomInterstitialAdsBinding dialogbinding = null;
        final Activity act;
        final String url;
        Bitmap bitmap;
        boolean isCustomChrome = false;
        JSONObject model;
        final DelegatingInterstitialAdLoadCallback listerner;

        CustomAdDialog(
                @NonNull Activity act,
                @NonNull String url,
                JSONObject model,
                @NonNull DelegatingInterstitialAdLoadCallback listerner
        ) {
            super(act, android.R.style.Theme_Black_NoTitleBar_Fullscreen);
            this.act = act;
            this.url = url;
            this.model = model;
            this.listerner = listerner;
        }

        void openCustomTab(String url) {
            CustomTabsIntent.Builder builder = new CustomTabsIntent.Builder();
            builder.setInstantAppsEnabled(false);
            builder.setUrlBarHidingEnabled(false);
            CustomTabsIntent customTabsIntent = builder.build();
            customTabsIntent.intent.setFlags(Intent.FLAG_ACTIVITY_NO_HISTORY);
            customTabsIntent.intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
            customTabsIntent.launchUrl(act, Uri.parse(url));
        }

        public void isCustomChrome(boolean is_cust) {
            isCustomChrome = is_cust;
        }

        @Override
        public void show() {
            if (isCustomChrome) {
                if (this.model.has("qureka_link")) {
                    try {
                        openCustomTab(this.model.getString("qureka_link"));
                        listerner.onInterstitialDismissed();
                        return;
                    } catch (JSONException e) { }
                }
                listerner.onFailedToShowFullScreenContent();
                return;
            }

            super.show();



            if (dialogbinding != null && bitmap != null) {
                Log.e(TAG, "load:    Step 7");
                dialogbinding.ivInterstitialImage.setImageBitmap(bitmap);
                dialogbinding.ivInterstitialImage.setVisibility(View.VISIBLE);
                dialogbinding.vwInterstitialWeb.setVisibility(View.GONE);
                dialogbinding.progressbar.setVisibility(View.GONE);
                listerner.onInterstitialDisplayed();
                return;
            }

            try {
                Log.e(TAG, "load:    Step 8" );
                if (dialogbinding != null && model != null && this.model.has("is_qureka_webview") && this.model.getBoolean("is_qureka_webview")) {
                    dialogbinding.vwInterstitialWeb.setVisibility(View.VISIBLE);
                    dialogbinding.progressbar.setVisibility(View.VISIBLE);
                    dialogbinding.ivInterstitialImage.setVisibility(View.GONE);
                    dialogbinding.btnClose.setVisibility(View.GONE);
                    if (this.model.has("qureka_webview_timing")) {
                        new Handler(Looper.getMainLooper()).postDelayed(() -> dialogbinding.btnClose.setVisibility(View.VISIBLE), this.model.getInt("qureka_webview_timing") * 1000L);
                    }

                    if ((this.model.has("qureka_webview_agent")) && !this.model.getString("qureka_webview_agent").isEmpty()) {
                        dialogbinding.webInterstitial.getSettings().setUserAgentString(this.model.getString("qureka_webview_agent"));
                    }

                    dialogbinding.webInterstitial.getSettings().setJavaScriptEnabled(true);
                    dialogbinding.webInterstitial.setLayerType(View.LAYER_TYPE_HARDWARE, null);
                    dialogbinding.webInterstitial.getSettings().setDomStorageEnabled(true);
                    dialogbinding.webInterstitial.getSettings().setSupportZoom(true);
                    dialogbinding.webInterstitial.getSettings().setBuiltInZoomControls(true);
                    dialogbinding.webInterstitial.getSettings().setDisplayZoomControls(false);
                    dialogbinding.webInterstitial.getSettings().setUseWideViewPort(true);
                    dialogbinding.webInterstitial.getSettings().setMixedContentMode(WebSettings.MIXED_CONTENT_ALWAYS_ALLOW);
                    dialogbinding.webInterstitial.getSettings().setMixedContentMode(WebSettings.MIXED_CONTENT_COMPATIBILITY_MODE);
                    dialogbinding.webInterstitial.getSettings().setJavaScriptCanOpenWindowsAutomatically(true);
                    dialogbinding.webInterstitial.getSettings().setLoadWithOverviewMode(true);
                    dialogbinding.webInterstitial.getSettings().setAllowFileAccess(true);

                    dialogbinding.webInterstitial.setWebViewClient(new WebViewClient() {
                        @Override
                        public void onPageFinished(WebView view, String url) {
                            super.onPageFinished(view, url);
                            dialogbinding.progressbar.setVisibility(View.GONE);
                        }

                        @Override
                        public boolean shouldOverrideUrlLoading(WebView view, WebResourceRequest request) {
                            Intent intent = new Intent(Intent.ACTION_VIEW, request.getUrl());
                            view.getContext().startActivity(intent);
                            return true;
                        }
                    });

                    dialogbinding.webInterstitial.setWebChromeClient(new WebChromeClient() {
                        @Override
                        public void onProgressChanged(WebView view, int newProgress) {
                            super.onProgressChanged(view, newProgress);
                            if (newProgress >= 40) {
                                dialogbinding.progressbar.setVisibility(View.GONE);
                            }
                        }


                    });

                    if ((this.model.has("qureka_webview_script")) && !this.model.getString("qureka_webview_script").isEmpty()) {
                        dialogbinding.webInterstitial.evaluateJavascript(this.model.getString("qureka_webview_script"), null);
                    }

                    if (this.model.has("qureka_link")) {
                        dialogbinding.webInterstitial.loadUrl(this.model.getString("qureka_link"));
                    }

                    return;

                }
            } catch (JSONException e) {
                Log.e(TAG, "load:    Step 10" );
                dismiss();
                listerner.onInterstitialDismissed();
                return;
            }

            listerner.onFailedToShowFullScreenContent();
        }

        @Override
        protected void onCreate(Bundle savedInstanceState) {
            super.onCreate(savedInstanceState);
            dialogbinding = CustomInterstitialAdsBinding.inflate(LayoutInflater.from(act));
            if (dialogbinding != null) {
                setContentView(dialogbinding.getRoot());
                dialogbinding.btnClose.setOnClickListener(view -> {
                    dismiss();
                    listerner.onInterstitialDismissed();
                });
                dialogbinding.progressbar.setVisibility(View.GONE);
                dialogbinding.ivInterstitialImage.setVisibility(View.VISIBLE);
                dialogbinding.vwInterstitialWeb.setVisibility(View.VISIBLE);
                dialogbinding.viewCustomIntra.setOnClickListener(view -> {
                    try {
                        if (this.model != null &&  this.model.has("qureka_link") && this.model.getString("qureka_link") != null && !this.model.getString("qureka_link").isEmpty()) {
                            if (this.model.has("is_qureka_custom_tab") && this.model.getBoolean("is_qureka_custom_tab")) {
                                openCustomTab(this.model.getString("qureka_link"));
                            } else {
                                openPage(this.model.getString("qureka_link"));
                                listerner.onAdClicked();
                            }
                        }
                    } catch (Exception e) {
                        listerner.onAdClicked();
                    }
                });
                dialogbinding.ivInterstitialImage.setOnClickListener(view -> {
                    try {
                        if (this.model != null &&  this.model.has("qureka_link") && this.model.getString("qureka_link") != null && !this.model.getString("qureka_link").isEmpty()) {
                            if (this.model.has("is_qureka_custom_tab") && this.model.getBoolean("is_qureka_custom_tab")) {
                                openCustomTab(this.model.getString("qureka_link"));
                            } else {
                                openPage(this.model.getString("qureka_link"));
                                listerner.onAdClicked();
                            }
                        }
                    } catch (Exception e) {
                        listerner.onAdClicked();
                    }
                });
            }

//            Log.e("WebViewActivity", "UA: " + dialogbinding.webInterstitial.getSettings().getUserAgentString());

            setCanceledOnTouchOutside(false);
            WindowManager.LayoutParams layoutParams = new WindowManager.LayoutParams();
            layoutParams.copyFrom(getWindow().getAttributes());
            layoutParams.width = WindowManager.LayoutParams.MATCH_PARENT;
            layoutParams.height = WindowManager.LayoutParams.MATCH_PARENT;
            getWindow().setAttributes(layoutParams);
            getWindow().setBackgroundDrawable(new ColorDrawable(android.graphics.Color.TRANSPARENT));
        }

        public void loadAd() {
            Log.e(TAG, "load:    Step 6  ------ "+this.url );
            Picasso.get().load(url).into(listerner);
        }

        public void setBitmap(Bitmap bitmap) {
            this.bitmap = bitmap;
        }

        private void openPage(String qurekaLink) {
            try {
                Intent browserIntent = new Intent(Intent.ACTION_VIEW, Uri.parse(qurekaLink));
                browserIntent.setPackage("com.android.chrome");
                act.startActivity(browserIntent);
            } catch (Exception exp) {
                try {
                    Intent browserIntent =
                            new Intent(Intent.ACTION_VIEW, Uri.parse(qurekaLink));
                    act.startActivity(browserIntent);
                } catch (Exception ex) {}
            }
        }
    }

    /**
     * An InterstitialAdLoadCallback that just forwards events to a delegate.
     */
    private static final class DelegatingInterstitialAdLoadCallback
            implements Target {

        private final WeakReference<FlutterCustomInterstitialAd> delegate;

        DelegatingInterstitialAdLoadCallback(FlutterCustomInterstitialAd delegate) {
            this.delegate = new WeakReference<>(delegate);
        }

        @Override
        public void onBitmapLoaded(Bitmap bitmap, Picasso.LoadedFrom from) {
            Log.e(TAG, "load:    Step 9" );
            if (delegate.get() != null) {
                delegate.get().onAdLoaded(bitmap);
            }
        }


        @Override
        public void onBitmapFailed(Exception e, Drawable errorDrawable) {
            Log.e(TAG, "load:    Step 10" );
            if (delegate.get() != null) {
                delegate.get().onAdFailedToLoad(new LoadAdError(101, e.getMessage(), "quereaka.com", null, null));
            }
        }

        @Override
        public void onPrepareLoad(Drawable placeHolderDrawable) {

        }

        public void onInterstitialDisplayed() {
            Log.e(TAG, "load:    Step 111" );
            if (delegate.get() != null) {
                delegate.get().manager.onAdShowedFullScreenContent(delegate.get().adId);
            }
        }
        public void onFailedToShowFullScreenContent() {
            Log.e(TAG, "load:    Step 11" );
            if (delegate.get() != null) {
                delegate.get().onFailedToShow();
            }
        }

        public void onInterstitialDismissed() {
            Log.e(TAG, "load:    Step 12" );
            if (delegate.get() != null) {
                delegate.get().manager.onAdDismissedFullScreenContent(delegate.get().adId);
            }
        }

        public void onAdClicked() {
            Log.e(TAG, "load:    Step 13" );
            if (delegate.get() != null) {
                delegate.get().manager.onAdClicked(delegate.get().adId);
            }
        }
    }
}