import 'dart:async';
import 'dart:convert';

import 'package:flutter/widgets.dart';
import 'package:google_applovin_unity_ads/ads/SettingsModel.dart';
import 'package:google_applovin_unity_ads/google_applovin_unity_ads.dart';
import 'package:url_launcher/url_launcher.dart';

const AdRequest _request = AdRequest(
    // keywords: <String>['foo', 'bar'],
    // contentUrl: 'http://foo.com/bar.html',
    // nonPersonalizedAds: true,
    );

const AdManagerAdRequest _ad_manager_request = AdManagerAdRequest(
    // keywords: <String>['foo', 'bar'],
    // contentUrl: 'http://foo.com/bar.html',
    // nonPersonalizedAds: true,
    );

NativeAd? _nativeAd;
AppLovinNativeAd? _appLovinnativeAd;
FacebookNativeAd? _facebooknativeAd;
CustomNativeAd? _customnativeAd;
AdManagerBannerAd? _adManagerBannerAd;
AppLovinBannerAd? _appLovinBannerAd;
int _TOTALDELAYD = 0;

const String _GOOGLEAD = "google_ads";
const String _ADMANAGERAD = "ad_manager";
const String _APPLOVINAD = "app_lovin";
const String _FACEBOOKAD = "facebook_ads";
const String _CUSTOMAD = "custom_ads";

Settings? _settings = getAdsSettings();
Map<String, dynamic>? _settingsJson = getAdsSettingsJson();
int appVersionCode = getAppVersionCode();

enum GradientOrientation {
  TOP_BOTTOM,
  TR_BL,
  RIGHT_LEFT,
  BR_TL,
  BOTTOM_TOP,
  BL_TR,
  LEFT_RIGHT,
  TL_BR
}

void setTimerDelayed() {
  _TOTALDELAYD = _settings?.adSetting?.adsLoadingTimeout ?? 1500;
}

void resetTimerDelayed() {
  _TOTALDELAYD = 0;
}

Map<String, dynamic> _getCurSeqData(List<String>? sequence, pos) {
  String? seqName = sequence?[pos];
  Map<String, dynamic> tempJson = {};
  tempJson["key_name"] = seqName;

  if (_settingsJson?.containsKey(seqName) == true) {
    if (seqName?.startsWith(_GOOGLEAD) == true ||
        seqName?.startsWith(_ADMANAGERAD) == true) {
      if (seqName?.startsWith(_GOOGLEAD) == true) {
        tempJson["seq_name"] = _GOOGLEAD;
      } else if (seqName?.startsWith(_ADMANAGERAD) == true) {
        tempJson["seq_name"] = _ADMANAGERAD;
      }
    } else if (seqName?.startsWith(_APPLOVINAD) == true) {
      tempJson["seq_name"] = _APPLOVINAD;
    } else if (seqName?.startsWith(_FACEBOOKAD) == true) {
      tempJson["seq_name"] = _FACEBOOKAD;
    } else if (seqName?.startsWith(_CUSTOMAD) == true) {
      tempJson["seq_name"] = _CUSTOMAD;
    }
    tempJson["json"] = _settingsJson?[seqName];
  }
  return tempJson;
}

void destroyAd() {
  getInterestitialAd()?.dispose();
  getAdManagerInterstitialAd()?.dispose();
  getFacebookInterestitialAd()?.dispose();
  getCustomInterestitialAd()?.dispose();
  getRewardedAd()?.dispose();
  getFacebookRewardedAd()?.dispose();
  getAppLovinRewardedAd()?.dispose();
  _adManagerBannerAd?.dispose();
  _appLovinBannerAd?.dispose();
  _facebooknativeAd?.dispose();
  _appLovinnativeAd?.dispose();
  _customnativeAd?.dispose();
  _nativeAd?.dispose();
}

initOpenAds({Function? onOpenAdLoaded, Function? onOpenAdFailedLoaded}) {
  if (_settings == null) {
    if (onOpenAdLoaded != null) {
      onOpenAdLoaded();
    }
    return;
  }

  if (_settings?.adSetting?.appVersionCode == appVersionCode) {
    if (onOpenAdLoaded != null) {
      onOpenAdLoaded();
    }
    return;
  }

  AppOpenAdManager appOpenAdManager = AppOpenAdManager()
    ..loadAd(callback: onOpenAdLoaded);

  setAppLifecycleReactor(
      AppLifecycleReactor(appOpenAdManager: appOpenAdManager));

  getAppLifecycleReactor()?.listenToAppStateChanges();
}

showOpenAds() {
  if (_settings == null) {
    return;
  }

  if (_settings?.adSetting?.appVersionCode == appVersionCode) {
    return;
  }

  if ((_settings?.isAllAds ?? false) == false) {
    return;
  }
  getAppLifecycleReactor()?.appOpenAdManager.showAdIfAvailable();
}

showIntraAds(
    {required BuildContext context,
    Function? callback,
    bool isCompulsory = false}) async {
  showProgressbarDialog(context);
  Timer? timer = null;
  timer = Timer(
      Duration(milliseconds: _settings?.adSetting?.adsTimeout ?? 3000), () {
    timer?.cancel();
    Navigator.pop(context);
    if (callback != null) {
      callback();
    }
  });

  loadIntraAds(
      isCompulsory: isCompulsory,
      onAdLoadedCallback: (isTrue) {
        if (timer?.isActive == true) {
          int remainingTime = isTrue
              ? (_settings?.adSetting?.adsLoadingTimeout ?? 1500) -
                  (timer?.tick ?? 0)
              : 0;
          if (remainingTime > 0) {
            Future.delayed(
                Duration(milliseconds: remainingTime),
                () => {
                      if (timer?.isActive == true)
                        {
                          timer?.cancel(),
                          Navigator.pop(context),
                          _showInterstitialAd(callback)
                        }
                    });
          } else {
            timer?.cancel();
            Navigator.pop(context);
            _showInterstitialAd(callback);
          }
        }
      });
}

showRewardAds(
    {required BuildContext context,
    Function? callback,
    Function? onFailedCallback,
    Function? onSkippedCallback}) async {
  showProgressbarDialog(context);
  resetTimerDelayed();
  Timer? timer;
  timer = Timer(
      Duration(milliseconds: _settings?.adSetting?.adsTimeout ?? 3000), () {
    timer?.cancel();
    Navigator.pop(context);
    if (onFailedCallback != null) {
      onFailedCallback();
    }
  });
  loadRewardAds(
      onAdLoadedCallback: () {
        if (timer?.isActive == true) {
          int remainingTime =
              (_settings?.adSetting?.adsLoadingTimeout ?? 1500) -
                  (timer?.tick ?? 0);
          if (remainingTime > 0) {
            Future.delayed(
                Duration(milliseconds: remainingTime),
                () => {
                      if (timer?.isActive == true)
                        {
                          timer?.cancel(),
                          Navigator.pop(context),
                          _showRewardedAd(
                              callback, onFailedCallback, onSkippedCallback)
                        }
                    });
          } else {
            if (timer?.isActive == true) {
              timer?.cancel();
              Navigator.pop(context);
              _showRewardedAd(callback, onFailedCallback, onSkippedCallback);
            }
          }
        }
      },
      onAdFailedCallback: () => {
            if (timer?.isActive == true)
              {
                Future.delayed(
                    Duration(milliseconds: _TOTALDELAYD),
                    () => {
                          if (timer?.isActive == true)
                            {
                              timer?.cancel(),
                              Navigator.pop(context),
                              if (onFailedCallback != null) {onFailedCallback()}
                            }
                        })
              }
          });
}

showBannerAds(
    {int pos = 0,
    int retry = 0,
    AdSize size = AdSize.banner,
    EdgeInsetsGeometry? margin,
    Decoration? decoration,
    Function(Widget)? onAdLoadedCallback,
    Function? onAdFailedToLoadCallback}) async {
  if (_settings == null ||
      (_settings?.adSetting?.appVersionCode == appVersionCode) ||
      (_settings?.bannerAdsSequence?.isEmpty ?? true) ||
      (_settings?.isAllAds != true)) {
    if (onAdFailedToLoadCallback != null) {
      onAdFailedToLoadCallback();
    }
    return;
  }

  if ((_settings?.bannerAdsSequence?.length ?? 0) <= pos) {
    if ((_settings?.adSetting?.adsRetry ?? 0) > 0 &&
        (_settings?.adSetting?.adsRetry ?? 0) > retry) {
      pos = 0;
      retry++;
    } else {
      resetTimerDelayed();
      if (onAdFailedToLoadCallback != null) {
        onAdFailedToLoadCallback();
      }
      return;
    }
  }

  incrementBannerSequence().then((value) {
    currentBannerSequence(pos).then((curSeqBanner) {
      pos = curSeqBanner % (_settings?.bannerAdsSequence?.length ?? 0);
      var ads_json = _getCurSeqData(_settings?.bannerAdsSequence, pos);
      var seqName =
          ads_json.containsKey("seq_name") ? ads_json["seq_name"] : "";

      switch (seqName) {
        case _GOOGLEAD:
          GoogleAds ads = GoogleAds.fromJson(ads_json["json"]);

          if ((ads.isBannerAds ?? false) && (ads.bannerId ?? "").isNotEmpty) {
            BannerAd(
                    size: size,
                    adUnitId: ads.bannerId ?? "",
                    listener: BannerAdListener(
                      onAdLoaded: (Ad ad) {
                        if (onAdLoadedCallback != null && ad is AdWithView) {
                          onAdLoadedCallback(Container(
                            width: size.width.toDouble(),
                            height: size.height.toDouble(),
                            margin: margin,
                            decoration: decoration,
                            child: AdWidget(ad: ad),
                          ));
                        }
                      },
                      onAdFailedToLoad: (Ad ad, LoadAdError error) {
                        ad.dispose();
                        showBannerAds(
                            retry: retry,
                            pos: pos + 1,
                            size: size,
                            onAdLoadedCallback: onAdLoadedCallback,
                            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
                      },
                      onAdOpened: (Ad ad) {},
                      onAdClosed: (Ad ad) {},
                    ),
                    request: _request)
                .load();
          } else {
            showBannerAds(
                retry: retry,
                pos: pos + 1,
                size: size,
                onAdLoadedCallback: onAdLoadedCallback,
                onAdFailedToLoadCallback: onAdFailedToLoadCallback);
          }
          break;
        case _ADMANAGERAD:
          GoogleAds ads = GoogleAds.fromJson(ads_json["json"]);
          if ((ads.isBannerAds ?? false) && (ads.bannerId ?? "").isNotEmpty) {
            AdManagerBannerAd(
              adUnitId: ads.bannerId ?? "",
              sizes: <AdSize>[size],
              request: _ad_manager_request,
              listener: AdManagerBannerAdListener(
                onAdLoaded: (Ad ad) async {
                  if (onAdLoadedCallback != null && ad is AdWithView) {
                    onAdLoadedCallback(Container(
                      width: size.width.toDouble(),
                      height: size.height.toDouble(),
                      margin: margin,
                      decoration: decoration,
                      child: AdWidget(ad: ad),
                    ));
                  }
                  // After the ad is loaded, get the platform ad size and use it to
                  // update the height of the container. This is necessary because the
                  // height can change after the ad is loaded.

                  /* AdManagerBannerAd bannerAd = (ad as AdManagerBannerAd);
              final AdSize? size = await bannerAd.getPlatformAdSize();

              if (size == null) {
                return;
              }*/
                },
                onAdFailedToLoad: (Ad ad, LoadAdError error) {
                  ad.dispose();
                  // _adManagerBannerAd = null;
                  showBannerAds(
                      retry: retry,
                      pos: pos + 1,
                      size: size,
                      onAdLoadedCallback: onAdLoadedCallback,
                      onAdFailedToLoadCallback: onAdFailedToLoadCallback);
                },
              ),
            ).load();
          } else {
            showBannerAds(
                retry: retry,
                pos: pos + 1,
                size: size,
                onAdLoadedCallback: onAdLoadedCallback,
                onAdFailedToLoadCallback: onAdFailedToLoadCallback);
          }
          break;
        case _APPLOVINAD:
          AppLovin ads = AppLovin.fromJson(ads_json["json"]);
          if ((ads.isBannerAds ?? false) && (ads.bannerId ?? "").isNotEmpty) {
            AppLovinBannerAd(
              adUnitId: size.height < 250
                  ? (ads.bannerId ?? "")
                  : ((ads.mrecId ?? "").isNotEmpty
                      ? (ads.mrecId ?? "")
                      : (ads.bannerId ?? "")),
              size: size,
              listener: BannerAdListener(
                onAdLoaded: (Ad ad) async {
                  if (onAdLoadedCallback != null && ad is AdWithView) {
                    onAdLoadedCallback(Container(
                      width: size.width.toDouble(),
                      height: size.height.toDouble(),
                      margin: margin,
                      decoration: decoration,
                      child: AdWidget(ad: ad),
                    ));
                  }
                },
                onAdFailedToLoad: (Ad ad, LoadAdError error) {
                  ad.dispose();
                  showBannerAds(
                      retry: retry,
                      pos: pos + 1,
                      size: size,
                      onAdLoadedCallback: onAdLoadedCallback,
                      onAdFailedToLoadCallback: onAdFailedToLoadCallback);
                },
              ),
            ).load();
          } else {
            showBannerAds(
                retry: retry,
                pos: pos + 1,
                size: size,
                onAdLoadedCallback: onAdLoadedCallback,
                onAdFailedToLoadCallback: onAdFailedToLoadCallback);
          }
          break;
        case _FACEBOOKAD:
          FacebookAd ads = FacebookAd.fromJson(ads_json["json"]);
          if ((ads.isBannerAds ?? false) && (ads.bannerId ?? "").isNotEmpty) {
            FacebookBannerAd(
              adUnitId: ads.bannerId ?? "",
              size: size,
              listener: BannerAdListener(
                onAdLoaded: (Ad ad) async {
                  if (onAdLoadedCallback != null && ad is AdWithView) {
                    onAdLoadedCallback(Container(
                      width: size.width.toDouble(),
                      height: size.height.toDouble(),
                      margin: margin,
                      decoration: decoration,
                      child: AdWidget(ad: ad),
                    ));
                  }
                },
                onAdFailedToLoad: (Ad ad, LoadAdError error) {
                  ad.dispose();
                  showBannerAds(
                      retry: retry,
                      pos: pos + 1,
                      size: size,
                      onAdLoadedCallback: onAdLoadedCallback,
                      onAdFailedToLoadCallback: onAdFailedToLoadCallback);
                },
              ),
            ).load();
          } else {
            showBannerAds(
                retry: retry,
                pos: pos + 1,
                size: size,
                onAdLoadedCallback: onAdLoadedCallback,
                onAdFailedToLoadCallback: onAdFailedToLoadCallback);
          }
          break;
        case _CUSTOMAD:
          CustomAds ads = CustomAds.fromJson(ads_json["json"]);
          if ((ads.isQurakaBanner ?? false) &&
              (ads.qurakaBannerImage?.isNotEmpty ?? false)) {
            final adUnitId = (ads.qurakaBannerImage?..shuffle())?.first ?? "";

            CustomBannerAd(
              adUnitId: adUnitId,
              size: size,
              listener: BannerAdListener(onAdLoaded: (Ad ad) async {
                if (onAdLoadedCallback != null && ad is AdWithView) {
                  onAdLoadedCallback(Container(
                    width: size.width.toDouble(),
                    height: size.height.toDouble(),
                    margin: margin,
                    decoration: decoration,
                    child: AdWidget(ad: ad),
                  ));
                }
              }, onAdFailedToLoad: (Ad ad, LoadAdError error) {
                ad.dispose();
                showBannerAds(
                    retry: retry,
                    pos: pos + 1,
                    size: size,
                    onAdLoadedCallback: onAdLoadedCallback,
                    onAdFailedToLoadCallback: onAdFailedToLoadCallback);
              }, onAdClicked: (Ad ad) {
                launchURL(ads.qurekaLink ?? "");
              }),
            ).load();
          } else {
            showBannerAds(
                retry: retry,
                pos: pos + 1,
                size: size,
                onAdLoadedCallback: onAdLoadedCallback,
                onAdFailedToLoadCallback: onAdFailedToLoadCallback);
          }
          break;
        default:
          showBannerAds(
              retry: retry,
              pos: pos + 1,
              size: size,
              onAdLoadedCallback: onAdLoadedCallback,
              onAdFailedToLoadCallback: onAdFailedToLoadCallback);
          break;
      }
    });
  });
}

showNativeAds(
    {int pos = 0,
    int retry = 0,
    bool isMediumNativeAds = false,
    bool isSequenceIgnore = false,
    Color backgroundColor = const Color(0xFFFFFFFF),
    Color borderColor = const Color(0xFFFFFFFF),
    double borderwidth = 1.0,
    BorderStyle borderstyle = BorderStyle.solid,
    EdgeInsetsGeometry? widgetMargin,
    double borderRadius = 12,
    Size size = const Size(double.infinity, 350),
    Map<String, Object>? customOptions,
    Function(Widget)? onAdLoadedCallback,
    Function? onAdFailedToLoadCallback}) async {
  if (_settings == null ||
      (_settings?.adSetting?.appVersionCode == appVersionCode) ||
      (_settings?.nativeAdsSequence?.isEmpty ?? true) ||
      (_settings?.isAllAds != true)) {
    if (onAdFailedToLoadCallback != null) {
      onAdFailedToLoadCallback();
    }
    return;
  }

  if ((_settings?.nativeAdsSequence?.length ?? 0) <= pos) {
    if ((_settings?.adSetting?.adsRetry ?? 0) > 0 &&
        (_settings?.adSetting?.adsRetry ?? 0) > retry) {
      pos = 0;
      retry++;
    } else {
      resetTimerDelayed();
      if (onAdFailedToLoadCallback != null) {
        onAdFailedToLoadCallback();
      }
      return;
    }
  }

  if (isSequenceIgnore) {
    _loadRequestNativeAd(
        retry,
        pos,
        isMediumNativeAds,
        isSequenceIgnore,
        backgroundColor,
        borderColor,
        borderwidth,
        borderstyle,
        widgetMargin,
        borderRadius,
        size,
        customOptions,
        onAdLoadedCallback,
        onAdFailedToLoadCallback);
  } else {
    incrementNativeSequence().then((value) {
      currentNativeSequence(pos).then((curSeqNative) {
        pos = curSeqNative % (_settings?.nativeAdsSequence?.length ?? 0);
        _loadRequestNativeAd(
            retry,
            pos,
            isMediumNativeAds,
            isSequenceIgnore,
            backgroundColor,
            borderColor,
            borderwidth,
            borderstyle,
            widgetMargin,
            borderRadius,
            size,
            customOptions,
            onAdLoadedCallback,
            onAdFailedToLoadCallback);
      });
    });
  }
}

_loadRequestNativeAd(
    int retry,
    int pos,
    bool isMediumNativeAds,
    bool isSequenceIgnore,
    Color backgroundColor,
    Color borderColor,
    double borderwidth,
    BorderStyle borderstyle,
    EdgeInsetsGeometry? widgetMargin,
    double borderRadius,
    Size size,
    Map<String, Object>? customOptions,
    Function(Widget)? onAdLoadedCallback,
    Function? onAdFailedToLoadCallback) {
  var ads_json = _getCurSeqData(_settings?.nativeAdsSequence, pos);

  var seqName = ads_json.containsKey("seq_name") ? ads_json["seq_name"] : "";
  switch (seqName) {
    case _GOOGLEAD:
      GoogleAds ads = GoogleAds.fromJson(ads_json["json"]);
      if ((ads.isNativeAds ?? false) && (ads.nativeId ?? "").isNotEmpty) {
        _nativeAd = NativeAd(
          adUnitId: ads.nativeId ?? "",
          isMediumNativeAds: isMediumNativeAds,
          request: _request,
          factoryId: 'nativeFactoryId',
          customOptions: customOptions,
          listener: NativeAdListener(
            onAdLoaded: (Ad nativeAd) {
              if (onAdLoadedCallback != null && nativeAd is AdWithView) {
                onAdLoadedCallback(Container(
                  width: size.width.toDouble(),
                  height: size.height.toDouble(),
                  margin: widgetMargin,
                  alignment: Alignment.center,
                  decoration: BoxDecoration(
                      color: backgroundColor,
                      border: Border.all(
                          color: borderColor,
                          width: borderwidth,
                          style: borderstyle),
                      borderRadius:
                          BorderRadius.all(Radius.circular(borderRadius))),
                  child: AdWidget(ad: nativeAd),
                ));
              }
            },
            onAdFailedToLoad: (Ad ad, LoadAdError error) {
              ad.dispose();
              _nativeAd = null;
              showNativeAds(
                  retry: retry,
                  pos: pos + 1,
                  size: size,
                  widgetMargin: widgetMargin,
                  backgroundColor: backgroundColor,
                  borderColor: borderColor,
                  borderwidth: borderwidth,
                  borderstyle: borderstyle,
                  borderRadius: borderRadius,
                  isMediumNativeAds: isMediumNativeAds,
                  customOptions: customOptions,
                  onAdLoadedCallback: onAdLoadedCallback,
                  onAdFailedToLoadCallback: onAdFailedToLoadCallback);
            },
            onAdOpened: (Ad ad) {},
            onAdClosed: (Ad ad) {},
          ),
        )..load();
      } else {
        showNativeAds(
            retry: retry,
            pos: pos + 1,
            size: size,
            widgetMargin: widgetMargin,
            isMediumNativeAds: isMediumNativeAds,
            backgroundColor: backgroundColor,
            borderColor: borderColor,
            borderwidth: borderwidth,
            borderstyle: borderstyle,
            borderRadius: borderRadius,
            customOptions: customOptions,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      }
      break;
    case _ADMANAGERAD:
      GoogleAds ads = GoogleAds.fromJson(ads_json["json"]);
      if ((ads.isNativeAds ?? false) && (ads.nativeId ?? "").isNotEmpty) {
        _nativeAd = NativeAd.fromAdManagerRequest(
            adUnitId: ads.nativeId ?? "",
            isMediumNativeAds: isMediumNativeAds,
            factoryId: 'nativeFactoryId',
            listener: NativeAdListener(
              onAdLoaded: (Ad nativeAd) {
                if (onAdLoadedCallback != null && nativeAd is AdWithView) {
                  onAdLoadedCallback(Container(
                    width: size.width.toDouble(),
                    height: size.height.toDouble(),
                    alignment: Alignment.center,
                    margin: widgetMargin,
                    decoration: BoxDecoration(
                        color: backgroundColor,
                        border: Border.all(
                            color: borderColor,
                            width: borderwidth,
                            style: borderstyle),
                        borderRadius:
                            BorderRadius.all(Radius.circular(borderRadius))),
                    child: AdWidget(ad: nativeAd),
                  ));
                }
              },
              onAdFailedToLoad: (Ad ad, LoadAdError error) {
                ad.dispose();
                _nativeAd = null;
                showNativeAds(
                    retry: retry,
                    pos: pos + 1,
                    size: size,
                    widgetMargin: widgetMargin,
                    backgroundColor: backgroundColor,
                    borderColor: borderColor,
                    borderwidth: borderwidth,
                    borderstyle: borderstyle,
                    borderRadius: borderRadius,
                    isMediumNativeAds: isMediumNativeAds,
                    customOptions: customOptions,
                    onAdLoadedCallback: onAdLoadedCallback,
                    onAdFailedToLoadCallback: onAdFailedToLoadCallback);
              },
              onAdOpened: (Ad ad) {},
              onAdClosed: (Ad ad) {},
            ),
            adManagerRequest: _ad_manager_request)
          ..load();
      } else {
        showNativeAds(
            retry: retry,
            pos: pos + 1,
            size: size,
            widgetMargin: widgetMargin,
            backgroundColor: backgroundColor,
            borderColor: borderColor,
            borderwidth: borderwidth,
            borderstyle: borderstyle,
            borderRadius: borderRadius,
            isMediumNativeAds: isMediumNativeAds,
            customOptions: customOptions,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      }
      break;
    case _APPLOVINAD:
      AppLovin ads = AppLovin.fromJson(ads_json["json"]);
      if ((ads.isNativeAds ?? false) &&
          (ads.nativeAds?.manual ?? "").isNotEmpty) {
        _appLovinnativeAd = AppLovinNativeAd(
            isMediumNativeAds: isMediumNativeAds,
            adUnitId: ads.nativeAds?.manual ?? "",
            factoryId: 'nativeFactoryId',
            customOptions: customOptions,
            listener: NativeAdListener(
              onAdLoaded: (Ad nativeAd) {
                if (onAdLoadedCallback != null && nativeAd is AdWithView) {
                  onAdLoadedCallback(Container(
                    width: size.width.toDouble(),
                    height: size.height.toDouble(),
                    alignment: Alignment.center,
                    margin: widgetMargin,
                    decoration: BoxDecoration(
                        color: backgroundColor,
                        border: Border.all(
                            color: borderColor,
                            width: borderwidth,
                            style: borderstyle),
                        borderRadius:
                            BorderRadius.all(Radius.circular(borderRadius))),
                    child: AdWidget(ad: nativeAd),
                  ));
                }
              },
              onAdFailedToLoad: (Ad ad, LoadAdError error) {
                ad.dispose();
                _appLovinnativeAd = null;
                showNativeAds(
                    retry: retry,
                    pos: pos + 1,
                    size: size,
                    widgetMargin: widgetMargin,
                    backgroundColor: backgroundColor,
                    borderColor: borderColor,
                    borderwidth: borderwidth,
                    borderstyle: borderstyle,
                    borderRadius: borderRadius,
                    isMediumNativeAds: isMediumNativeAds,
                    customOptions: customOptions,
                    onAdLoadedCallback: onAdLoadedCallback,
                    onAdFailedToLoadCallback: onAdFailedToLoadCallback);
              },
              onAdOpened: (Ad ad) {},
              onAdClosed: (Ad ad) {},
            ))
          ..load();
      } else {
        showNativeAds(
            retry: retry,
            pos: pos + 1,
            size: size,
            widgetMargin: widgetMargin,
            isMediumNativeAds: isMediumNativeAds,
            backgroundColor: backgroundColor,
            borderColor: borderColor,
            borderwidth: borderwidth,
            borderstyle: borderstyle,
            borderRadius: borderRadius,
            customOptions: customOptions,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      }
      break;
    case _FACEBOOKAD:
      FacebookAd ads = FacebookAd.fromJson(ads_json["json"]);
      if ((ads.isNativeAds ?? false) && (ads.nativeId ?? "").isNotEmpty) {
        _facebooknativeAd = FacebookNativeAd(
            isMediumNativeAds: isMediumNativeAds,
            adUnitId: ads.nativeId ?? "",
            factoryId: 'nativeFactoryId',
            customOptions: customOptions,
            listener: NativeAdListener(
              onAdLoaded: (Ad nativeAd) async {
                if (onAdLoadedCallback != null && nativeAd is AdWithView) {
                  onAdLoadedCallback(Container(
                    width: size.width.toDouble(),
                    height: size.height.toDouble(),
                    alignment: Alignment.center,
                    margin: widgetMargin,
                    decoration: BoxDecoration(
                        color: backgroundColor,
                        border: Border.all(
                            color: borderColor,
                            width: borderwidth,
                            style: borderstyle),
                        borderRadius:
                            BorderRadius.all(Radius.circular(borderRadius))),
                    child: AdWidget(ad: nativeAd),
                  ));
                }
              },
              onAdFailedToLoad: (Ad ad, LoadAdError error) {
                ad.dispose();
                _facebooknativeAd = null;
                showNativeAds(
                    retry: retry,
                    pos: pos + 1,
                    size: size,
                    widgetMargin: widgetMargin,
                    backgroundColor: backgroundColor,
                    borderColor: borderColor,
                    borderwidth: borderwidth,
                    borderstyle: borderstyle,
                    borderRadius: borderRadius,
                    isMediumNativeAds: isMediumNativeAds,
                    customOptions: customOptions,
                    onAdLoadedCallback: onAdLoadedCallback,
                    onAdFailedToLoadCallback: onAdFailedToLoadCallback);
              },
              onAdOpened: (Ad ad) {},
              onAdClosed: (Ad ad) {},
            ))
          ..load();
      } else {
        showNativeAds(
            retry: retry,
            pos: pos + 1,
            size: size,
            widgetMargin: widgetMargin,
            isMediumNativeAds: isMediumNativeAds,
            backgroundColor: backgroundColor,
            borderColor: borderColor,
            borderwidth: borderwidth,
            borderstyle: borderstyle,
            borderRadius: borderRadius,
            customOptions: customOptions,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      }
      break;
    case _CUSTOMAD:
      CustomAds ads = CustomAds.fromJson(ads_json["json"]);
      if ((ads.isQurakaNative ?? false) &&
          (ads.qurakaNativeImage?.isNotEmpty ?? false)) {
        final adUnitId = (ads.qurakaNativeImage?..shuffle())?.first ?? "";
        _customnativeAd = CustomNativeAd(
            isMediumNativeAds: isMediumNativeAds,
            adUnitId: adUnitId,
            factoryId: 'nativeFactoryId',
            listener: NativeAdListener(
                onAdLoaded: (Ad nativeAd) {
                  if (onAdLoadedCallback != null && nativeAd is AdWithView) {
                    onAdLoadedCallback(Container(
                      width: size.width.toDouble(),
                      height: size.height.toDouble(),
                      alignment: Alignment.center,
                      margin: widgetMargin,
                      decoration: BoxDecoration(
                          color: backgroundColor,
                          border: Border.all(
                              color: borderColor,
                              width: borderwidth,
                              style: borderstyle),
                          borderRadius:
                              BorderRadius.all(Radius.circular(borderRadius))),
                      child: AdWidget(ad: nativeAd),
                    ));
                  }
                },
                onAdFailedToLoad: (Ad ad, LoadAdError error) {
                  ad.dispose();
                  _customnativeAd = null;
                  showNativeAds(
                      retry: retry,
                      pos: pos + 1,
                      size: size,
                      widgetMargin: widgetMargin,
                      backgroundColor: backgroundColor,
                      borderColor: borderColor,
                      borderwidth: borderwidth,
                      borderstyle: borderstyle,
                      borderRadius: borderRadius,
                      isMediumNativeAds: isMediumNativeAds,
                      customOptions: customOptions,
                      onAdLoadedCallback: onAdLoadedCallback,
                      onAdFailedToLoadCallback: onAdFailedToLoadCallback);
                },
                onAdOpened: (Ad ad) {},
                onAdClosed: (Ad ad) {},
                onAdClicked: (Ad ad) => {launchURL(ads.qurekaLink ?? "")}))
          ..load();
      } else {
        showNativeAds(
            retry: retry,
            pos: pos + 1,
            size: size,
            widgetMargin: widgetMargin,
            isMediumNativeAds: isMediumNativeAds,
            backgroundColor: backgroundColor,
            borderColor: borderColor,
            borderwidth: borderwidth,
            borderstyle: borderstyle,
            borderRadius: borderRadius,
            customOptions: customOptions,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      }
      break;
    default:
      showNativeAds(
          retry: retry,
          pos: pos + 1,
          size: size,
          widgetMargin: widgetMargin,
          isMediumNativeAds: isMediumNativeAds,
          backgroundColor: backgroundColor,
          borderColor: borderColor,
          borderwidth: borderwidth,
          borderstyle: borderstyle,
          borderRadius: borderRadius,
          customOptions: customOptions,
          onAdLoadedCallback: onAdLoadedCallback,
          onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      break;
  }
}

showMediumNativeAds(
    {Size size = const Size(double.infinity, 160),
    Color backgroundColor = const Color(0xFFFFFFFF),
    Color borderColor = const Color(0xFFFFFFFF),
    double borderwidth = 1.0,
    BorderStyle borderstyle = BorderStyle.solid,
    double borderRadius = 12,
    EdgeInsetsGeometry? widgetMargin,
    Map<String, Object>? customOptions,
    Function(Widget)? onAdLoadedCallback,
    Function? onAdFailedToLoadCallback}) async {
  showNativeAds(
      pos: 0,
      size: size,
      widgetMargin: widgetMargin,
      isMediumNativeAds: true,
      backgroundColor: backgroundColor,
      borderColor: borderColor,
      borderwidth: borderwidth,
      borderstyle: borderstyle,
      borderRadius: borderRadius,
      customOptions: customOptions,
      onAdLoadedCallback: onAdLoadedCallback,
      onAdFailedToLoadCallback: onAdFailedToLoadCallback);
}

_showNativeFixedAds(String nativeAdsKey, bool defaultOtherNativeAds,
    {Size size = const Size(double.infinity, 160),
    Color backgroundColor = const Color(0xFFFFFFFF),
    Color borderColor = const Color(0xFFFFFFFF),
    double borderwidth = 1.0,
    bool isMediumNativeAds = false,
    BorderStyle borderstyle = BorderStyle.solid,
    double borderRadius = 12,
    EdgeInsetsGeometry? widgetMargin,
    Map<String, Object>? customOptions,
    Function(Widget)? onAdLoadedCallback,
    Function? onAdFailedToLoadCallback}) async {
  if (getAdsSettings()?.nativeAdsSequence?.contains(nativeAdsKey) == true) {
    int customPos =
        getAdsSettings()?.nativeAdsSequence?.indexOf(nativeAdsKey) ?? -1;
    showNativeAds(
        pos: customPos,
        size: size,
        isSequenceIgnore: true,
        widgetMargin: widgetMargin,
        isMediumNativeAds: isMediumNativeAds,
        backgroundColor: backgroundColor,
        borderColor: borderColor,
        borderwidth: borderwidth,
        borderstyle: borderstyle,
        borderRadius: borderRadius,
        customOptions: customOptions,
        onAdLoadedCallback: onAdLoadedCallback,
        onAdFailedToLoadCallback: onAdFailedToLoadCallback);
  } else if (defaultOtherNativeAds) {
    showNativeAds(
        pos: 0,
        size: size,
        widgetMargin: widgetMargin,
        isMediumNativeAds: true,
        backgroundColor: backgroundColor,
        borderColor: borderColor,
        borderwidth: borderwidth,
        borderstyle: borderstyle,
        borderRadius: borderRadius,
        customOptions: customOptions,
        onAdLoadedCallback: onAdLoadedCallback,
        onAdFailedToLoadCallback: onAdFailedToLoadCallback);
  } else {
    if (onAdFailedToLoadCallback != null) {
      onAdFailedToLoadCallback();
    }
  }
}

showNativeCustomAds(
    {Size size = const Size(double.infinity, 160),
    Color backgroundColor = const Color(0xFFFFFFFF),
    Color borderColor = const Color(0xFFFFFFFF),
    double borderwidth = 1.0,
    bool isMediumNativeAds = false,
    bool defaultOtherNativeAds = false,
    BorderStyle borderstyle = BorderStyle.solid,
    double borderRadius = 12,
    EdgeInsetsGeometry? widgetMargin,
    Map<String, Object>? customOptions,
    Function(Widget)? onAdLoadedCallback,
    Function? onAdFailedToLoadCallback}) {
  _showNativeFixedAds(_CUSTOMAD, defaultOtherNativeAds,
      size: size,
      widgetMargin: widgetMargin,
      isMediumNativeAds: isMediumNativeAds,
      backgroundColor: backgroundColor,
      borderColor: borderColor,
      borderwidth: borderwidth,
      borderstyle: borderstyle,
      borderRadius: borderRadius,
      customOptions: customOptions,
      onAdLoadedCallback: onAdLoadedCallback,
      onAdFailedToLoadCallback: onAdFailedToLoadCallback);
}

showNativeGoogleAds(
    {Size size = const Size(double.infinity, 160),
    Color backgroundColor = const Color(0xFFFFFFFF),
    Color borderColor = const Color(0xFFFFFFFF),
    double borderwidth = 1.0,
    bool isMediumNativeAds = false,
    bool defaultOtherNativeAds = false,
    BorderStyle borderstyle = BorderStyle.solid,
    double borderRadius = 12,
    Map<String, Object>? customOptions,
    EdgeInsetsGeometry? widgetMargin,
    Function(Widget)? onAdLoadedCallback,
    Function? onAdFailedToLoadCallback}) {
  _showNativeFixedAds(_GOOGLEAD, defaultOtherNativeAds,
      size: size,
      widgetMargin: widgetMargin,
      isMediumNativeAds: isMediumNativeAds,
      backgroundColor: backgroundColor,
      borderColor: borderColor,
      borderwidth: borderwidth,
      borderstyle: borderstyle,
      borderRadius: borderRadius,
      customOptions: customOptions,
      onAdLoadedCallback: onAdLoadedCallback,
      onAdFailedToLoadCallback: onAdFailedToLoadCallback);
}

showNativeAppLovinAds(
    {Size size = const Size(double.infinity, 160),
    Color backgroundColor = const Color(0xFFFFFFFF),
    Color borderColor = const Color(0xFFFFFFFF),
    double borderwidth = 1.0,
    bool isMediumNativeAds = false,
    bool defaultOtherNativeAds = false,
    BorderStyle borderstyle = BorderStyle.solid,
    double borderRadius = 12,
    EdgeInsetsGeometry? widgetMargin,
    Map<String, Object>? customOptions,
    Function(Widget)? onAdLoadedCallback,
    Function? onAdFailedToLoadCallback}) {
  _showNativeFixedAds(_APPLOVINAD, defaultOtherNativeAds,
      size: size,
      widgetMargin: widgetMargin,
      isMediumNativeAds: isMediumNativeAds,
      backgroundColor: backgroundColor,
      borderColor: borderColor,
      borderwidth: borderwidth,
      borderstyle: borderstyle,
      borderRadius: borderRadius,
      customOptions: customOptions,
      onAdLoadedCallback: onAdLoadedCallback,
      onAdFailedToLoadCallback: onAdFailedToLoadCallback);
}

showNativeFacebookAds(
    {Size size = const Size(double.infinity, 160),
    Color backgroundColor = const Color(0xFFFFFFFF),
    Color borderColor = const Color(0xFFFFFFFF),
    double borderwidth = 1.0,
    bool isMediumNativeAds = false,
    bool defaultOtherNativeAds = false,
    BorderStyle borderstyle = BorderStyle.solid,
    double borderRadius = 12,
    EdgeInsetsGeometry? widgetMargin,
    Map<String, Object>? customOptions,
    Function(Widget)? onAdLoadedCallback,
    Function? onAdFailedToLoadCallback}) {
  _showNativeFixedAds(_FACEBOOKAD, defaultOtherNativeAds,
      size: size,
      widgetMargin: widgetMargin,
      isMediumNativeAds: isMediumNativeAds,
      backgroundColor: backgroundColor,
      borderColor: borderColor,
      borderwidth: borderwidth,
      borderstyle: borderstyle,
      borderRadius: borderRadius,
      customOptions: customOptions,
      onAdLoadedCallback: onAdLoadedCallback,
      onAdFailedToLoadCallback: onAdFailedToLoadCallback);
}

IsFullIntraAds({required BuildContext context, Function? callback}) {
  if (_settings?.adSetting?.isFullAds ?? false) {
    showIntraAds(context: context, callback: callback);
    return;
  }
  if (callback != null) {
    callback();
  }
}

showFullNativeAds(
    {Size? size,
    Color backgroundColor = const Color(0xFFFFFFFF),
    Color borderColor = const Color(0xFFFFFFFF),
    double borderwidth = 1.0,
    BorderStyle borderstyle = BorderStyle.solid,
    double borderRadius = 12,
    EdgeInsetsGeometry? widgetMargin,
    Map<String, Object>? customOptions,
    Function(Widget)? onAdLoadedCallback,
    Function? onAdFailedToLoadCallback}) {
  if (_settings?.adSetting?.isFullAds ?? false) {
    (size != null)
        ? showNativeAds(
            pos: 0,
            size: size,
            isMediumNativeAds: true,
            widgetMargin: widgetMargin,
            backgroundColor: backgroundColor,
            borderColor: borderColor,
            borderwidth: borderwidth,
            borderstyle: borderstyle,
            borderRadius: borderRadius,
            customOptions: customOptions,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback)
        : showNativeAds(
            backgroundColor: backgroundColor,
            borderColor: borderColor,
            widgetMargin: widgetMargin,
            borderwidth: borderwidth,
            borderstyle: borderstyle,
            borderRadius: borderRadius,
            customOptions: customOptions,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
    return;
  }

  if (onAdLoadedCallback != null) {
    onAdLoadedCallback(Container(height: 0));
  }
}

showFullMediumNativeAds(
    {Size? size,
    Color backgroundColor = const Color(0xFFFFFFFF),
    Color borderColor = const Color(0xFFFFFFFF),
    double borderwidth = 1.0,
    EdgeInsetsGeometry? widgetMargin,
    BorderStyle borderstyle = BorderStyle.solid,
    double borderRadius = 12,
    Map<String, Object>? customOptions,
    Function(Widget)? onAdLoadedCallback,
    Function? onAdFailedToLoadCallback}) {
  if (_settings?.adSetting?.isFullAds ?? false) {
    (size != null)
        ? showMediumNativeAds(
            size: size,
            backgroundColor: backgroundColor,
            borderColor: borderColor,
            borderwidth: borderwidth,
            borderstyle: borderstyle,
            borderRadius: borderRadius,
            widgetMargin: widgetMargin,
            customOptions: customOptions,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback)
        : showMediumNativeAds(
            backgroundColor: backgroundColor,
            borderColor: borderColor,
            borderwidth: borderwidth,
            widgetMargin: widgetMargin,
            borderstyle: borderstyle,
            borderRadius: borderRadius,
            customOptions: customOptions,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
    return;
  }

  if (onAdLoadedCallback != null) {
    onAdLoadedCallback(Container(height: 0));
  }
}

void loadIntraAds(
    {int pos = 0,
    int retry = 0,
    bool isCompulsory = false,
    Function(bool)? onAdLoadedCallback}) async {
  if (_settings == null) {
    resetTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback(false);
    }
    return;
  }

  if (_settings?.intraAdsSequence?.isEmpty ?? true) {
    resetTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback(false);
    }
    return;
  }

  if (_settings?.adSetting?.appVersionCode == appVersionCode) {
    resetTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback(false);
    }
    return;
  }

  if ((_settings?.intraAdsSequence?.length ?? 0) <= pos) {
    if ((_settings?.adSetting?.adsRetry ?? 0) > 0 &&
        (_settings?.adSetting?.adsRetry ?? 0) > retry) {
      pos = 0;
      retry++;
    } else {
      resetTimerDelayed();
      if (onAdLoadedCallback != null) {
        onAdLoadedCallback(true);
      }
      return;
    }
  }

  if (_settings?.isAllAds != true) {
    resetTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback(false);
    }
    return;
  }

  final curSeqIntra = await currentIntraSequence(pos);
  pos = curSeqIntra % (_settings?.intraAdsSequence?.length ?? 0);
  final isInterestial = ((_settings?.adSetting?.interestialAdsInterval ?? 0) >
              0 &&
          (_settings?.adSetting?.interestialAdsIntervalCount ?? 0) %
                  (_settings?.adSetting?.interestialAdsInterval ?? 0) ==
              0 &&
          ((getInterestialAdTimeInterval() +
                      ((_settings?.adSetting?.interestialAdsTimingInterval ??
                              0) *
                          1000)) <=
                  DateTime.now().millisecondsSinceEpoch ||
              getInterestialAdTimeInterval() == 0)) ||
      isCompulsory;

  if (!isInterestial) {
    resetTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback(false);
    }
    return;
  }

  var ads_json = _getCurSeqData(_settings?.intraAdsSequence, pos);

  var seqName = ads_json.containsKey("seq_name") ? ads_json["seq_name"] : "";
  switch (seqName) {
    case _GOOGLEAD:
    case _ADMANAGERAD:
      GoogleAds ads = GoogleAds.fromJson(ads_json["json"]);
      if ((ads.isInterstitialAds ?? true) &&
          (ads.interestialId ?? "").isNotEmpty) {
        if (_ADMANAGERAD == seqName) {
          _createAdManagerInterstitialAd(
              pos, isCompulsory, retry, ads, onAdLoadedCallback);
          return;
        }
        _createInterstitialAd(
            pos, isCompulsory, retry, ads, onAdLoadedCallback);
        return;
      }
      await incrementIntraSequence();
      loadIntraAds(
          retry: retry,
          pos: pos + 1,
          isCompulsory: isCompulsory,
          onAdLoadedCallback: onAdLoadedCallback);
      break;
    case _APPLOVINAD:
      AppLovin ads = AppLovin.fromJson(ads_json["json"]);
      if ((ads.isInterstitialAds ?? true) &&
          (ads.interestialId ?? "").isNotEmpty) {
        // incrementIntraSequnce();
        _createAppLovinInterstitialAd(
            pos, isCompulsory, retry, ads, onAdLoadedCallback);
        return;
      }
      await incrementIntraSequence();
      loadIntraAds(
          retry: retry,
          pos: pos + 1,
          isCompulsory: isCompulsory,
          onAdLoadedCallback: onAdLoadedCallback);
      break;
    case _FACEBOOKAD:
      FacebookAd ads = FacebookAd.fromJson(ads_json["json"]);
      if ((ads.isInterstitialAds ?? true) &&
          (ads.interestialId ?? "").isNotEmpty) {
        // incrementIntraSequnce();
        _createFacebookInterstitialAd(
            pos, isCompulsory, retry, ads, onAdLoadedCallback);
        return;
      }
      await incrementIntraSequence();
      loadIntraAds(
          retry: retry,
          pos: pos + 1,
          isCompulsory: isCompulsory,
          onAdLoadedCallback: onAdLoadedCallback);
      break;
    case _CUSTOMAD:
      CustomAds ads = CustomAds.fromJson(ads_json["json"]);
      _createCustomInterstitialAd(pos, isCompulsory, retry, ads,
          json.encode(ads_json["json"]), onAdLoadedCallback);
      return;
    default:
      await incrementIntraSequence();
      loadIntraAds(
          retry: retry,
          pos: pos + 1,
          isCompulsory: isCompulsory,
          onAdLoadedCallback: onAdLoadedCallback);
      break;
  }
}

void loadRewardAds(
    {int pos = 0,
    int retry = 0,
    Function? onAdLoadedCallback,
    Function? onAdFailedCallback}) async {
  if (_settings == null) {
    resetTimerDelayed();
    if (onAdFailedCallback != null) {
      onAdFailedCallback();
    }
    return;
  }

  if (_settings?.rewardsAdsSequence?.isEmpty ?? true) {
    resetTimerDelayed();
    if (onAdFailedCallback != null) {
      onAdFailedCallback();
    }
    return;
  }

  if (_settings?.adSetting?.appVersionCode == appVersionCode) {
    resetTimerDelayed();
    if (onAdFailedCallback != null) {
      onAdFailedCallback();
    }
    return;
  }

  if ((_settings?.rewardsAdsSequence?.length ?? 0) <= pos) {
    if ((_settings?.adSetting?.adsRetry ?? 0) > 0 &&
        (_settings?.adSetting?.adsRetry ?? 0) > retry) {
      pos = 0;
      retry++;
    } else {
      resetTimerDelayed();
      if (onAdLoadedCallback != null) {
        onAdLoadedCallback();
      }
      return;
    }
  }

  if (_settings?.isAllAds != true) {
    resetTimerDelayed();
    if (onAdFailedCallback != null) {
      onAdFailedCallback();
    }
    return;
  }

  final curSeqReward = await currentRewardSequence(pos);

  pos = curSeqReward % (_settings?.rewardsAdsSequence?.length ?? 0);

  var ads_json = _getCurSeqData(_settings?.rewardsAdsSequence, pos);

  var seqName = ads_json.containsKey("seq_name") ? ads_json["seq_name"] : "";

  switch (seqName) {
    case _GOOGLEAD:
    case _ADMANAGERAD:
      GoogleAds ads = GoogleAds.fromJson(ads_json["json"]);
      if ((ads.isRewaredAds ?? false) && (ads.rewardedId ?? "").isNotEmpty) {
        if (_ADMANAGERAD == seqName) {
          _createAdmanagerRewardedAd(pos, retry, onAdLoadedCallback,
              ads.rewardedId ?? "", onAdFailedCallback);
          return;
        }
        _createRewardedAd(pos, retry, onAdLoadedCallback, ads.rewardedId ?? "",
            onAdFailedCallback);
        return;
      } else {
        await incrementRewardSequence();
        loadRewardAds(
            pos: pos + 1,
            retry: retry,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedCallback: onAdFailedCallback);
      }
      break;
    case _APPLOVINAD:
      AppLovin ads = AppLovin.fromJson(ads_json["json"]);
      if ((ads.isRewaredAds ?? false) && (ads.rewaredId ?? "").isNotEmpty) {
        _createAppLovinRewardedAd(pos, retry, onAdLoadedCallback,
            ads.rewaredId ?? "", onAdFailedCallback);
        return;
      } else {
        await incrementRewardSequence();
        loadRewardAds(
            retry: retry,
            pos: pos + 1,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedCallback: onAdFailedCallback);
      }
      break;
    case _FACEBOOKAD:
      FacebookAd ads = FacebookAd.fromJson(ads_json["json"]);
      if ((ads.isRewaredAds ?? false) && (ads.rewardedId ?? "").isNotEmpty) {
        _createFacebookRewardedAd(pos, retry, onAdLoadedCallback,
            ads.rewardedId ?? "", onAdFailedCallback);
        return;
      } else {
        await incrementRewardSequence();
        loadRewardAds(
            retry: retry,
            pos: pos + 1,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedCallback: onAdFailedCallback);
      }
      break;
    default:
      await incrementRewardSequence();
      loadRewardAds(
          retry: retry,
          pos: pos + 1,
          onAdLoadedCallback: onAdLoadedCallback,
          onAdFailedCallback: onAdFailedCallback);
      break;
  }
}

void _createInterstitialAd(int pos, bool isCompulsory, int retry, GoogleAds ads,
    Function(bool)? onAdLoadedCallback) async {
  if (getInterestitialAd() != null) {
    setTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback(true);
    }
    return;
  }
  resetTimerDelayed();
  InterstitialAd.load(
      adUnitId: ads.interestialId ?? "",
      request: _request,
      adLoadCallback: InterstitialAdLoadCallback(
        onAdLoaded: (InterstitialAd ad) async {
          setInterestitialAd(ad);
          getInterestitialAd()?.setImmersiveMode(true);
          setIntraCustomImpression(ads.isRedirectCustomImpression ?? false);
          if (onAdLoadedCallback != null) {
            onAdLoadedCallback(true);
          }
        },
        onAdFailedToLoad: (LoadAdError error) async {
          setInterestitialAd(null);
          await incrementIntraSequence();
          loadIntraAds(
              retry: retry,
              pos: pos + 1,
              isCompulsory: isCompulsory,
              onAdLoadedCallback: onAdLoadedCallback);
        },
      ));
}

void _createAppLovinInterstitialAd(int pos, bool isCompulsory, int retry,
    AppLovin ads, Function(bool)? onAdLoadedCallback) async {
  if (getAppLovinInterestitialAd() != null) {
    setTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback(true);
    }
    return;
  }
  resetTimerDelayed();
  AppLovinInterstitialAd.load(
      adUnitId: ads.interestialId ?? "",
      adLoadCallback: AppLovinInterstitialAdLoadCallback(
        onAdLoaded: (AppLovinInterstitialAd ad) {
          setAppLovinInterestitialAd(ad);
          setIntraCustomImpression(ads.isRedirectCustomImpression ?? false);
          if (onAdLoadedCallback != null) {
            onAdLoadedCallback(true);
          }
        },
        onAdFailedToLoad: (LoadAdError error) async {
          setAppLovinInterestitialAd(null);
          await incrementIntraSequence();
          loadIntraAds(
              retry: retry,
              pos: pos + 1,
              isCompulsory: isCompulsory,
              onAdLoadedCallback: onAdLoadedCallback);
        },
      ));
}

void _createFacebookInterstitialAd(int pos, bool isCompulsory, int retry,
    FacebookAd ads, Function(bool)? onAdLoadedCallback) async {
  if (getFacebookInterestitialAd() != null) {
    setTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback(true);
    }
    return;
  }
  resetTimerDelayed();
  FacebookInterstitialAd.load(
      adUnitId: ads.interestialId ?? "",
      adLoadCallback: FacebookInterstitialAdLoadCallback(
        onAdLoaded: (FacebookInterstitialAd ad) {
          setFacebookInterestitialAd(ad);
          setIntraCustomImpression(ads.isRedirectCustomImpression ?? false);
          if (onAdLoadedCallback != null) {
            onAdLoadedCallback(true);
          }
        },
        onAdFailedToLoad: (LoadAdError error) async {
          setFacebookInterestitialAd(null);
          await incrementIntraSequence();
          loadIntraAds(
              retry: retry,
              pos: pos + 1,
              isCompulsory: isCompulsory,
              onAdLoadedCallback: onAdLoadedCallback);
        },
      ));
}

void _createCustomInterstitialAd(
    int pos,
    bool isCompulsory,
    int retry,
    CustomAds ads,
    String custom_model,
    Function(bool)? onAdLoadedCallback) async {
  if (getCustomInterestitialAd() != null) {
    setTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback(true);
    }
    return;
  }
  resetTimerDelayed();

  String adUnitId = (ads.qurakaIntraImage?..shuffle())?.first ?? "";
  CustomInterstitialAd.load(
      adUnitId: adUnitId,
      custom_ads_model: custom_model,
      adLoadCallback: CustomInterstitialAdLoadCallback(
        onAdLoaded: (CustomInterstitialAd ad) {
          setCustomInterestitialAd(ad);
          setIntraCustomImpression(ads.isRedirectCustomImpression ?? false);
          if (onAdLoadedCallback != null) {
            onAdLoadedCallback(true);
          }
        },
        onAdFailedToLoad: (LoadAdError error) async {
          setCustomInterestitialAd(null);
          await incrementIntraSequence();
          loadIntraAds(
              retry: retry,
              pos: pos + 1,
              isCompulsory: isCompulsory,
              onAdLoadedCallback: onAdLoadedCallback);
        },
      ));
}

void _createRewardedAd(int pos, int retry, Function? onAdLoadedCallback,
    String rewardedId, Function? onAdFailedCallback) async {
  if (getRewardedAd() != null) {
    setTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }
  resetTimerDelayed();

  RewardedAd.load(
      adUnitId: rewardedId,
      request: _request,
      rewardedAdLoadCallback: RewardedAdLoadCallback(
        onAdLoaded: (RewardedAd ad) {
          setRewardedAd(ad);
          if (onAdLoadedCallback != null) {
            onAdLoadedCallback();
          }
        },
        onAdFailedToLoad: (LoadAdError error) async {
          await incrementRewardSequence();
          setRewardedAd(null);
          loadRewardAds(
              retry: retry,
              pos: pos + 1,
              onAdLoadedCallback: onAdLoadedCallback,
              onAdFailedCallback: onAdFailedCallback);
        },
      ));
}

void _createAdmanagerRewardedAd(
    int pos,
    int retry,
    Function? onAdLoadedCallback,
    String rewardedId,
    Function? onAdFailedCallback) async {
  if (getRewardedAd() != null) {
    setTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }
  resetTimerDelayed();
  RewardedAd.loadWithAdManagerAdRequest(
      adUnitId: rewardedId,
      adManagerRequest: _ad_manager_request,
      rewardedAdLoadCallback: RewardedAdLoadCallback(
        onAdLoaded: (RewardedAd ad) async {
          setRewardedAd(ad);
          if (onAdLoadedCallback != null) {
            onAdLoadedCallback();
          }
        },
        onAdFailedToLoad: (LoadAdError error) async {
          setRewardedAd(null);
          await incrementRewardSequence();
          loadRewardAds(
              retry: retry,
              pos: pos + 1,
              onAdLoadedCallback: onAdLoadedCallback,
              onAdFailedCallback: onAdFailedCallback);
        },
      ));
}

void _createAppLovinRewardedAd(int pos, int retry, Function? onAdLoadedCallback,
    String rewardedId, Function? onAdFailedCallback) async {
  if (getAppLovinRewardedAd() != null) {
    setTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }
  resetTimerDelayed();

  AppLovinRewardedAd.load(
      adUnitId: rewardedId,
      rewardedAdLoadCallback: AppLovinRewardedAdLoadCallback(
        onAdLoaded: (AppLovinRewardedAd ad) {
          setAppLovinRewardedAd(ad);
          if (onAdLoadedCallback != null) {
            onAdLoadedCallback();
          }
        },
        onAdFailedToLoad: (LoadAdError error) async {
          setAppLovinRewardedAd(null);
          await incrementRewardSequence();
          loadRewardAds(
              retry: retry,
              pos: pos + 1,
              onAdLoadedCallback: onAdLoadedCallback,
              onAdFailedCallback: onAdFailedCallback);
        },
      ));
}

void _createFacebookRewardedAd(int pos, int retry, Function? onAdLoadedCallback,
    String rewardedId, Function? onAdFailedCallback) async {
  if (getFacebookRewardedAd() != null) {
    setTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }
  resetTimerDelayed();
  FacebookRewardedAd.load(
      adUnitId: rewardedId,
      rewardedAdLoadCallback: FacebookRewardedAdLoadCallback(
        onAdLoaded: (FacebookRewardedAd ad) {
          setFacebookRewardedAd(ad);
          if (onAdLoadedCallback != null) {
            onAdLoadedCallback();
          }
        },
        onAdFailedToLoad: (LoadAdError error) async {
          await incrementRewardSequence();
          setFacebookRewardedAd(null);
          loadRewardAds(
              retry: retry,
              pos: pos + 1,
              onAdLoadedCallback: onAdLoadedCallback,
              onAdFailedCallback: onAdFailedCallback);
        },
      ));
}

void _createAdManagerInterstitialAd(int pos, bool isCompulsory, int retry,
    GoogleAds ads, Function(bool)? onAdLoadedCallback) async {
  if (getAdManagerInterstitialAd() != null) {
    setTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback(true);
    }
    return;
  }
  resetTimerDelayed();
  AdManagerInterstitialAd.load(
      adUnitId: ads.interestialId ?? "",
      request: _ad_manager_request,
      adLoadCallback: AdManagerInterstitialAdLoadCallback(
          onAdLoaded: (AdManagerInterstitialAd ad) {
        setAdManagerInterstitialAd(ad);
        getAdManagerInterstitialAd()?.setImmersiveMode(true);
        setIntraCustomImpression(ads.isRedirectCustomImpression ?? false);
        if (onAdLoadedCallback != null) {
          onAdLoadedCallback(true);
        }
      }, onAdFailedToLoad: (LoadAdError error) async {
        setAdManagerInterstitialAd(null);
        await incrementIntraSequence();
        loadIntraAds(
            retry: retry,
            pos: pos + 1,
            isCompulsory: isCompulsory,
            onAdLoadedCallback: onAdLoadedCallback);
      }));
}

void _showInterstitialAd(Function? callback) async {
  if (_settings == null || _settings?.isAllAds != true) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getInterestitialAd() == null) {
    _showAdManagerInterstitialAd(callback);
    return;
  }

  getInterestitialAd()?.fullScreenContentCallback = FullScreenContentCallback(
    onAdShowedFullScreenContent: (InterstitialAd ad) => {
      incrementIntraAdsCustomImpressionCount(IsIntraCustomImpression()),
    },
    onAdDismissedFullScreenContent: (InterstitialAd ad) async {
      ad.dispose();

      incrementCounter();
      _showCustomImpression(IsIntraCustomImpression());
      await incrementIntraSequence();
      setInterestitialAd(null);
      setInterestialAdTimeInterval(DateTime.now().millisecondsSinceEpoch);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
    onAdFailedToShowFullScreenContent: (InterstitialAd ad, AdError error) {
      ad.dispose();
      // _loadInterstitialAd(pos: pos + 1);
      setInterestitialAd(null);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
  );
  getInterestitialAd()?.show();
  setInterestitialAd(null);
}

void _showAdManagerInterstitialAd(Function? callback) async {
  if (_settings == null || _settings?.isAllAds != true) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getAdManagerInterstitialAd() == null) {
    _showAppLovinInterstitialAd(callback);
    return;
  }

  getAdManagerInterstitialAd()?.fullScreenContentCallback =
      FullScreenContentCallback(
    onAdShowedFullScreenContent: (AdManagerInterstitialAd ad) => {
      incrementIntraAdsCustomImpressionCount(IsIntraCustomImpression()),
    },
    onAdDismissedFullScreenContent: (AdManagerInterstitialAd ad) async {
      ad.dispose();
      incrementCounter();
      _showCustomImpression(IsIntraCustomImpression());
      await incrementIntraSequence();
      setInterestialAdTimeInterval(DateTime.now().millisecondsSinceEpoch);
      setAdManagerInterstitialAd(null);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
    onAdFailedToShowFullScreenContent:
        (AdManagerInterstitialAd ad, AdError error) {
      ad.dispose();
      // _loadInterstitialAd(pos: pos + 1);
      setAdManagerInterstitialAd(null);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
  );
  getAdManagerInterstitialAd()?.show();
  setAdManagerInterstitialAd(null);
}

void _showAppLovinInterstitialAd(Function? callback) async {
  if (_settings == null || _settings?.isAllAds != true) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getAppLovinInterestitialAd() == null) {
    _showFacebookInterstitialAd(callback);
    return;
  }

  getAppLovinInterestitialAd()?.fullScreenContentCallback =
      FullScreenContentCallback(
    onAdShowedFullScreenContent: (AppLovinInterstitialAd ad) => {
      incrementIntraAdsCustomImpressionCount(IsIntraCustomImpression()),
    },
    onAdDismissedFullScreenContent: (AppLovinInterstitialAd ad) async {
      ad.dispose();
      incrementCounter();
      _showCustomImpression(IsIntraCustomImpression());
      await incrementIntraSequence();
      setAppLovinInterestitialAd(null);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
    onAdFailedToShowFullScreenContent:
        (AppLovinInterstitialAd ad, AdError error) {
      ad.dispose();
      // _loadInterstitialAd(pos: pos + 1);
      setAppLovinInterestitialAd(null);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
  );
  getAppLovinInterestitialAd()?.show();
  setAppLovinInterestitialAd(null);
}

void _showFacebookInterstitialAd(Function? callback) async {
  if (_settings == null || _settings?.isAllAds != true) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getFacebookInterestitialAd() == null) {
    _showCustomInterstitialAd(callback);
    return;
  }

  getFacebookInterestitialAd()?.fullScreenContentCallback =
      FullScreenContentCallback(
    onAdShowedFullScreenContent: (FacebookInterstitialAd ad) => {
      incrementIntraAdsCustomImpressionCount(IsIntraCustomImpression()),
    },
    onAdDismissedFullScreenContent: (FacebookInterstitialAd ad) async {
      ad.dispose();
      await incrementIntraSequence();
      incrementCounter();
      _showCustomImpression(IsIntraCustomImpression());
      setFacebookInterestitialAd(null);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
    onAdFailedToShowFullScreenContent:
        (FacebookInterstitialAd ad, AdError error) {
      ad.dispose();
      // _loadInterstitialAd(pos: pos + 1);
      setFacebookInterestitialAd(null);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
  );
  getFacebookInterestitialAd()?.show();
  setFacebookInterestitialAd(null);
}

void _showCustomInterstitialAd(Function? callback) async {
  if (_settings == null || _settings?.isAllAds != true) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getCustomInterestitialAd() == null) {
    if (callback != null) {
      callback();
    }
    incrementCounter();
    loadIntraAds();
    return;
  }

  getCustomInterestitialAd()?.fullScreenContentCallback =
      FullScreenContentCallback(
          onAdShowedFullScreenContent: (CustomInterstitialAd ad) => {
                incrementIntraAdsCustomImpressionCount(
                    IsIntraCustomImpression()),
              },
          onAdDismissedFullScreenContent: (CustomInterstitialAd ad) async {
            ad.dispose();
            incrementCounter();
            _showCustomImpression(IsIntraCustomImpression());
            await incrementIntraSequence();
            setCustomInterestitialAd(null);
            if (callback != null) {
              callback();
            }
            loadIntraAds();
          },
          onAdFailedToShowFullScreenContent:
              (CustomInterstitialAd ad, AdError error) {
            ad.dispose();
            // _loadInterstitialAd(pos: pos + 1);
            setCustomInterestitialAd(null);
            if (callback != null) {
              callback();
            }
            loadIntraAds();
          },
          onAdClicked: (CustomInterstitialAd ad) {
            // launchURL(_settings?.customAds?.qurekaLink ?? "");
          });
  getCustomInterestitialAd()?.show();
  setCustomInterestitialAd(null);
}

void _showCustomImpression(bool isLocalImpression) {
  if ((_settings?.adSetting?.interestialAdsCustomImpressionInterval ?? 0) > 0 &&
      _settings?.customAds?.qurekaLink?.isNotEmpty == true &&
      isLocalImpression) {
    if (currentIntraAdsCustomImpression() %
            (_settings?.adSetting?.interestialAdsCustomImpressionInterval ??
                0) ==
        0) {
      launchURL(_settings?.customAds?.qurekaLink ?? "");
    }
  }
}

void _showRewardedAd(Function? callback, Function? onFailedCallback,
    Function? onSkippedCallback) async {
  if (_settings == null || _settings?.isAllAds != true) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getRewardedAd() == null) {
    _showAppLovinRewardedAd(callback, onFailedCallback, onSkippedCallback);
    return;
  }

  getRewardedAd()?.fullScreenContentCallback = FullScreenContentCallback(
    onAdShowedFullScreenContent: (RewardedAd ad) {},
    onAdDismissedFullScreenContent: (RewardedAd ad) async {
      ad.dispose();
      await incrementRewardSequence();
      loadRewardAds();
      if (IsRewardShowCompleted()) {
        if (callback != null) {
          callback();
        }
      } else {
        if (onSkippedCallback != null) {
          onSkippedCallback();
        }
      }
    },
    onAdFailedToShowFullScreenContent: (RewardedAd ad, AdError error) {
      ad.dispose();
      if (onFailedCallback != null) {
        onFailedCallback();
      }
      loadRewardAds();
    },
  );
  getRewardedAd()?.setImmersiveMode(true);
  setRewardShowCompleted(false);
  getRewardedAd()?.show(
      onUserEarnedReward: (AdWithoutView ad, RewardItem reward) {
    setRewardShowCompleted(true);
  });
  setRewardedAd(null);
}

void _showAppLovinRewardedAd(Function? callback, Function? onFailedCallback,
    Function? onSkippedCallback) async {
  if (_settings == null || _settings?.isAllAds != true) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getAppLovinRewardedAd() == null) {
    _showFacebookRewardedAd(callback, onFailedCallback, onSkippedCallback);
    return;
  }

  getAppLovinRewardedAd()?.fullScreenContentCallback =
      FullScreenContentCallback(
    onAdShowedFullScreenContent: (AppLovinRewardedAd ad) {},
    onAdDismissedFullScreenContent: (AppLovinRewardedAd ad) async {
      ad.dispose();
      await incrementRewardSequence();
      loadRewardAds();
      if (IsRewardShowCompleted()) {
        if (callback != null) {
          callback();
        }
      } else {
        if (onSkippedCallback != null) {
          onSkippedCallback();
        }
      }
    },
    onAdFailedToShowFullScreenContent: (AppLovinRewardedAd ad, AdError error) {
      ad.dispose();
      if (onFailedCallback != null) {
        onFailedCallback();
      }
      loadRewardAds();
    },
  );

  setRewardShowCompleted(false);
  getAppLovinRewardedAd()?.show(
      onUserEarnedReward: (AdWithoutView ad, RewardItem reward) {
    setRewardShowCompleted(true);
  });

  setAppLovinRewardedAd(null);
}

void _showFacebookRewardedAd(Function? callback, Function? onFailedCallback,
    Function? onSkippedCallback) async {
  if (_settings == null || _settings?.isAllAds != true) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getFacebookRewardedAd() == null) {
    if (onFailedCallback != null) {
      onFailedCallback();
    }
    return;
  }

  getFacebookRewardedAd()?.fullScreenContentCallback =
      FullScreenContentCallback(
    onAdShowedFullScreenContent: (FacebookRewardedAd ad) {},
    onAdDismissedFullScreenContent: (FacebookRewardedAd ad) async {
      ad.dispose();
      await incrementRewardSequence();
      loadRewardAds();
      if (IsRewardShowCompleted()) {
        if (callback != null) {
          callback();
        }
      } else {
        if (onSkippedCallback != null) {
          onSkippedCallback();
        }
      }
    },
    onAdFailedToShowFullScreenContent: (FacebookRewardedAd ad, AdError error) {
      ad.dispose();
      if (onFailedCallback != null) {
        onFailedCallback();
      }
      loadRewardAds();
    },
  );

  setRewardShowCompleted(false);
  getFacebookRewardedAd()?.show(
      onUserEarnedReward: (AdWithoutView ad, RewardItem reward) {
    setRewardShowCompleted(true);
  });

  setFacebookRewardedAd(null);
}

void launchURL(String url) async {
  try {
    if (url.isNotEmpty) {
      final uri = Uri.parse(url);
      if (await canLaunchUrl(uri)) {
        await launch(url);
      }
    }
  } catch (e) {}
}

void setProgressDialogWidget(Widget? wdt) {
  setProgressWidget(wdt);
}
